/**********************************************************************************
!C

!File: dec2bin.c

!Description:
  convert a decimal number to binary string 

!Input Parameters: (none)

!Input/Output Parameters: (none)

!Output Parameters: (none)

!Revision History:
  Original July 1999. QA Tool Version 1.2
  Modified on 3/3/2000: To include bit number as option

!Team-unique Header:
  This software was developed by:
    Land Data Operational Product Evaluation (LDOPE) Team for the
    National Aeronautics and Space Administration, Goddard Space Flight
    Center

!References and Credits:
    Sadashiva Devadiga
    LDOPE                             Science Systems and Applications Inc.
    devadiga@ltpmail.gsfc.nasa.gov    NASA/GSFC Code 922
    phone: 301-614-5449               Greenbelt, MD 20771

!Design Notes: (none)

!END
*********************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "qa_tool.h"
#include "str_op.h"
#include "main_util.h"

#define HELP \
"NAME \n" \
"       dec2bin - convert a signed decimal number to binary string \n" \
"\n" \
"SYNOPSIS \n" \
"       dec2bin [-help] \n" \
"       dec2bin <number> [-bn=<bit number|bit number range>]\n" \
" \n" \
"       echo <number> [-bn=<bit number|bit number range>] | dec2bin \n" \
" \n" \
"DESCRIPTION \n" \
"       This routine converts a signed decimal number to equivalent binary \n" \
"Output is in 32 bit binary form. Optional bit number argument can be used \n" \
"to print only the required bit or range of bits. \n" \
" \n" \
"OPTIONS \n" \
"       -help                       Display this help message \n" \
"       -bn=<bit number>            Zero-based bit number or bit number range. \n" \
"       number                      Number in decimal \n" \
" \n" \
"EXAMPLES \n" \
"	dec2bin 25 \n" \
"	dec2bin 101 \n" \
"	dec2bin 101 -b=5 \n" \
"	dec2bin 255 -bn=0-7\n" \
" \n" \
"Last update: Version 3.6, 02/12/2020 \n" \
"Please report problems to Sadashiva Devadiga (sadashiva.devadiga-1@nasa.gov) \n"

#define USAGE \
"usage:		dec2bin <number> [-bn=<bit number|bit number range>]\n" \
" \n" \
"OPTIONS \n" \
"       -help                       Display this help message \n" \
"       -bn=<bit number>            Zero-based bit number or bit number range. \n" \
"       number                      Number in decimal \n" \
" \n"

int parse_cmd_dec2bin(char **argv, int argc, int *num, int *bit_range);

int main(int argc, char **argv)
{
  int i, j;
  int k = 1;
  int bn_cnt;
  int num, num1;
  int bit_range[2];
  char bit_str[20];
  char num_str[40];

  if ((argc == 2) && ((strcmp(argv[1], "-h") == 0) || (strcmp(argv[1], "-help") == 0)))
  {
    fprintf(stderr, "%s\n", HELP);
    return 1;
  } 
  else if (parse_cmd_dec2bin(argv, argc, &num, bit_range) == -1)
  {
    fprintf(stderr, "Usage error . . . .  \n\n");
    fprintf(stderr, "%s\n", USAGE);
  }
  else
  {
    for (i=0, j=31; i<32; i++, j--)
    {
      num1 = num & k;
      num_str[j] = (num1 == 0) ? '0' : '1';
      k <<= 1; 
    }
  }
  num_str[i] = '\0';
  if (bit_range[0] == -1)
    fprintf(stdout, "%s\n", num_str);
  else
  {
    strrev(num_str);
    bn_cnt = bit_range[1] - bit_range[0] + 1;
    strmid(num_str, bit_range[0], bn_cnt, bit_str);
    strrev(bit_str);
    fprintf(stdout, "%s\n", bit_str);
  }
  return 1;
}
  
int parse_cmd_dec2bin(char **argv, int argc, int *num, int *bit_range)
{
  char bn_str[5];
  char range_str[10];
  int i, p1, st, len;

  range_str[0] = '\0';
  for (i=1, st = -1; i<argc; i++)
  {
    if (is_arg_id(argv[i], "-bn=") == 0)
      get_arg_val(argv[i], range_str);
/*
    else if (argv[i][0] == '-')
      fprintf(stderr, "Ignoring invalid option %s\n", argv[i]);
*/
    else
    {
      *num = atoi(argv[i]);
      st = 1;
    }
  }
  if (st == -1)
    fprintf(stderr, "Missing input decimal number \n");
  else if (range_str[0] != '\0')
  {
    if ((p1 = charpos(range_str, '-', 0)) != -1)
    {
      len = (int)strlen(range_str);
      strmid(range_str, 0, p1, bn_str);
      bit_range[0] = (int)atoi(bn_str);
      p1++;
      strmid(range_str, p1, len-p1, bn_str);
      bit_range[1] = (int)atoi(bn_str);
    }
    else
      bit_range[0] = bit_range[1] = (int)atoi(range_str);
  }
  else
    bit_range[0] = -1;
  return st;
}


