#include <stdio.h>
#include <stdlib.h>
#include "cproj.h"

#define MAXZONES 21600

static double R;
static double lon_center;
static double false_easting;
static double false_northing;
static double lat_step;
static double xratio[MAXZONES];
static double offset[MAXZONES];


long isinusint(double r, double center_long, double false_east, double false_north, int nzones, int nrows)
{
int izone, ncol;

R = r;
lon_center = center_long;
false_easting = false_east;
false_northing = false_north;
lat_step = 180. / nzones;

if (nzones % 2 != 0) {
  fprintf(stderr, "Number of zones (%d) is not a multiple of 2 in isinusint().\n", nzones);
  exit(-1);
}
if (nzones > MAXZONES) {
  fprintf(stderr, "Number of zones (%d) is greater than %d in isinusint().\n", nzones, MAXZONES);
  exit(-1);
}

for (izone=0; izone<nzones/2; izone++) {
  ncol = cos(HALF_PI * (1 - (izone + 0.5) * 2 / nzones)) * nrows + 0.5;
  if (ncol < 1) ncol = 1;
  xratio[izone] = R * ncol / nrows;
  if (ncol % 2 == 1) offset[izone] = 0.5 * R * 180. / nrows;
               else  offset[izone] = 0;
  offset[nzones - 1 - izone] = offset[izone];
  xratio[nzones - 1 - izone] = xratio[izone];
}

return OK;

}



#if 1
long isinusfor(double lon, double lat, double *x, double *y)
{
int izone;

if (fabs(lat) > 90) return ERROR;
izone = (90 - lat) / lat_step;
*x = lon * xratio[izone];
*x -= offset[izone];
*y = R * lat;
return OK;
}



long isinusinv(double x, double y, double *lon, double *lat)
{
int izone;

*lat = y / R;
if (fabs(*lat) >= 90) return ERROR;
izone = (90 - *lat) / lat_step;
x += offset[izone];
*lon = x / xratio[izone];
if (fabs(*lon) > 180) return ERROR;
return OK;
}

#endif
