#include "hdf.h"
#include "hdf5.h"
#include "hdf5_hl.h"
#include "qa_tool.h"
#define MAX_LDOPE_DIMS  4

typedef struct
{
  int H5IN;   /* derived by code for the input sds_t. no effect on the output sds_t */
  int H5OUT;  /* specified by users for the output sds_t. no effect on the input sds_t */
  char *file_name;  /* MSZ 8/17/17, to track the file name of the sds. Note that don't free it because it is not calloced just points to the first pointer of HDF_name */
  int32 use_L2G_max_nobs;/* MSZ 8/18/17, 1 is yes and 0 is no. For L2G compact sds only */
  size_t L2G_max_nobs;   /* MSZ 8/18/17, max_nobs for a line of L2G compact sds */
  char name[MAX_SDS_NAME_LEN];
  int32 data_type;       /* use HDF4 system */
  int32 data_size;
  int32 rank;
  int32 dim_size[MAX_LDOPE_DIMS];
  int32 row_id;          /* MSZ 2/05/2018, add row_id as an element of sds_t, 0-based */
  int32 interleave;      /* MSZ 2/05/2018, add band interleave as an element of sds_t. 1 for bsq and 0 for bip, just for rank>=2 */
  int32 non_xy_dims_size;/* MSZ 5/21/2019, add for non_xy_dims_size, and it is the 1 or multiplication of all non_XY dims. just for rank>=2 */
  int32 nattr;
  int fill_val;
  float64 fill_fval;
  unsigned fill_uval;
  int range[2];
  float32 frange[2];
  int32 scale; 
  float64 fscale;  
  int32 offset;
  float64 foffset;  
  int32 fscale_type;  
  int32 sd_id;
  int32 sds_id;
  int32 sds_index;
  /* HDF5 addition */
  hid_t gid;
  char h5gname[1024];   /* group name for the name */
  hid_t h5dmtype;
  hid_t h5dtype;
  hid_t h5space_id;
} sds_t;

typedef struct
{
  /* MSZ 11/05/2018 increase the size */
  char name[MAX_SDS_NAME_LEN];
  int32 type;
  int32 cnt;
  void *buf;
} sds_attr_t;

/* added by Maosheng Zhao for function HDF4_scan_SDSs_info(), HDF4_check_L2G() and HDF4_free_SDSs_info() 
 * July, 2017
 * Science Systems and Applications, Inc.
 */ 
typedef struct  /* on the file level, MSZ */
{
   int32 sd_id;
   int32 n_SDSs;    /* control the vector length of the below pointer elements except **SDS_attrs */
   char **SDS_names;
   int32 *data_type;
   int32 *rank;
   int32 **dim_sizes;
   int32 *n_attrs;  /* control length of first dim of below **SDS_attrs */
   sds_attr_t **SDS_attrs;   /* haven't supported yet in HDF4_scan_SDSs_info() */
} HDF4_SDSs_INFO;

typedef struct
{
    hsize_t num_objs_links;
    char **obj_name;
    H5O_type_t *obj_type;   /* H5O_type_t in H5Opublic.h. In most cases, just H5O_TYPE_GROUP (0) or H5O_TYPE_DATASET (1) */
} H5_OBJS_TYPES;
