/****************************************************************************
!C

!File: str_op.c

!Description:
  Contains routines for various string manipulation. 

!Input Parameters: (none)

!Input/Output Parameters: (none)

!Output Parameters: (none)

!Revision History:
  Original January/February/March 1998. Version 1.0
  Modified September 1999. Fixed a bug in strcmp_wc 
  Modified April 2000. get_sdsname_dim: (if sds name contains '(' character consider this SDS as 2D)
  Modified April 2019. Maosheng Zhao added strpos_wb() to make sure that the searched string segement is an
          an independent word or has word boundary
 
!Team-unique Header:
  This software was developed by:
    Land Data Operational Product Evaluation (LDOPE) Team for the
    National Aeronautics and Space Administration, Goddard Space Flight
    Center

!References and Credits:

    Sadashiva Devadiga
    LDOPE                             Science Systems and Applications Inc.
    devadiga@ltpmail.gsfc.nasa.gov    NASA/GSFC Code 922
    phone: 301-614-5449               Greenbelt, MD 20771

!Design Notes: (none)

!END
*****************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#include "qa_tool.h"
#include "str_op.h"

int charpos(char *s, char c, int p)
{
  int len, i;

  len = (int)strlen(s);
  for (i=p; i<len; i++)
    if (s[i] == c) return i;
  return -1;
}

int strpos(char *s1, char *s2, int p)
{
  int i, j, k, fn;
  int len1, len2;
  char *tmp = NULL;

  tmp=(char*)calloc(MAX_STR_LEN, sizeof(char));

  fn = 0;
  len1 = (int)strlen(s1);
  len2 = (int)strlen(s2);
  if (len1 >= len2)
    for (i = p; i <= len1-len2; i++)
    {
      if (s1[i] == s2[0]) 
      {
        for (j = i, k = 0; k < len2; k++, j++)
         tmp[k] = s1[j];
        tmp[k] = '\0'; 
        if (strcmp(tmp, s2) == 0) 
        {
          fn = 1;
          break;
        }
      }
    }

  free(tmp);

  if (fn == 1) return i;
    else return -1; 
}

int strcasepos(char *s1, char *s2, int p)
{
  int i, j, k, fn;
  int len1, len2;
  char *tmp = NULL;

  tmp=(char*)calloc(MAX_STR_LEN, sizeof(char));

  fn = 0;
  len1 = (int)strlen(s1);
  len2 = (int)strlen(s2);
  if (len1 >= len2)
    for (i = p; i <= len1-len2; i++)
    {
      if (s1[i] == s2[0])
/*
      if ((s1[i] == s2[0]) || (s1[i] == tolower(s2[0])) || (s2[0] == tolower(s1[i]))) 
*/
      {
        for (j = i, k = 0; k < len2; k++, j++)
         tmp[k] = s1[j];
        tmp[k] = '\0'; 
        if (strcasecmp(tmp, s2) == 0) 
        {
          fn = 1;
          break;
        }
      }
    }

  free(tmp);

  if (fn == 1) return i;
    else return -1; 
}

void strmid(char *s1, int p1, int cnt, char *s2)
{
  int i, j, p2;

  p2 = p1 + cnt;
  for (i=p1, j=0; i<p2; i++, j++)
    s2[j] = s1[i];
  s2[j] = '\0';
}

void strtrim(char *s)
{
  int i, j1, j2, k, len;

  len = (int)strlen(s);
  for (j1=0; j1<len; j1++)
    if ((s[j1] != ' ') && (s[j1] != '\t') && (s[j1] != '\n')) break;
  for (j2=len-1; j2>=0; j2--)
    if ((s[j2] != ' ') && (s[j2] != '\t') && (s[j1] != '\n')) break;
  for (i=j1, k=0; i<=j2; i++, k++)
    s[k] = s[i];
  s[k] = '\0';
}

void strrev(char *s)
{
  int i, k, len;
  char *s1 = NULL;

  s1=(char*)calloc(MAX_STR_LEN, sizeof(char));

  len = (int)strlen(s);
  strcpy(s1, s);
  for (i=len-1, k=0; i>=0; i--, k++)
    s[k] = s1[i];
  s[k] = '\0';

  free(s1);
}

void rm_ln_in_str(char *s)
{
  int i, j;
  int len;

  len = (int)strlen(s);
  for (i=0; i<len; i++)
    if (s[i] == '\n')
    {
      for (j=i+1; j<len; j++)
        s[j-1] = s[j];
      s[j-1] = '\0';
      len--; 
    }
}

/* Maosheng Zhao, SSAI, 3/12/2018 */
/* can remove any specific character c in the sring s, such
 * as '(', ')' ' ' etc */
void rm_character_in_str(char *s, char c)
{
  int i, j;
  int len;

  len = (int)strlen(s);
  for (i=0; i<len; i++)
    if (s[i] == c)
    {
      for (j=i+1; j<len; j++)
        s[j-1] = s[j];
      s[j-1] = '\0';
      len--; 
    }
}


int strcmp_wc(char *s1, char *s2)
/* searches for s2 in s1. s1 must be complete */
{
  char s_tmp[40];
  int p_s1, p_s2;
  int pt1, pt2, pt;
  int len_s1, len_s2;
  int end_len, l_tmp;
  char end_str_s1[25];
  char end_str_s2[25];

  len_s1 = (int)strlen(s1);
  len_s2 = (int)strlen(s2);
  for (p_s1=0, p_s2=0; ((p_s1 < len_s1) && (p_s2 < len_s2)); )
  {
    if ((s2[p_s2] == s1[p_s1]) || (s2[p_s2] == '?'))
    {
      p_s1++; p_s2++;
    }
    else if (s2[p_s2] == '*')
    {
      p_s2++;
      pt1 = charpos(s2, '*', p_s2);
      pt2 = charpos(s2, '?', p_s2);
      if ((pt1 == -1) && (pt2 == -1))
      {
	p_s1++;
	end_len = len_s2 - p_s2;
	strmid(s2, p_s2, end_len, end_str_s2);
	strmid(s1, len_s1-end_len, end_len, end_str_s1);
	if (strcmp(end_str_s1, end_str_s2) == 0)
	{
	  p_s1 = len_s1;
	  p_s2 = len_s2;
	}
	break;
      }
      else
      {
        if (pt1 == -1) pt = pt2;
        else if (pt2 == -1) pt = pt1;
        else pt = (pt1 > pt2) ? pt2 : pt1;
        strmid(s2, p_s2, pt-p_s2, s_tmp);
        p_s1 = strpos(s1, s_tmp, p_s1);
        if (p_s1 == -1) break;
        else
        {
          l_tmp = (int)strlen(s_tmp);
          p_s1 += l_tmp;
          p_s2 += l_tmp;
        }
      }
    }
    else break;
  }
  if ((p_s1 == len_s1) && (p_s2 == len_s2)) return 0;
  else return -1;
}

void sort_strings(char **s, int n)
{
  int i,j;
  char tmp[MAX_PATH_LENGTH];

  for (i=0; i<n-1; i++)
    for (j=n-1; j>i; j--)
      if (strcmp(s[j], s[j-1]) < 0)
      {
	strcpy(tmp, s[j]);
	strcpy(s[j], s[j-1]);
	strcpy(s[j-1], tmp);
      }    
  return;
}

int get_sdsname_dim(char *sdsname_str, char *sds_name, int *n, int *m)
{
  /* MSZ 5/27/2019. my understanding is that for >=3D data, if specify 1-based two two 
   * non-XY dimensional layers, n and m, in the -sds= argument */
  int st = 1;
  int p1, p2, len;
  char nstr[10] = {'\0'};

  *n = *m = -1;
  if (charpos(sdsname_str, '(', 0) == -1)
  {
    if ((p1 = charpos(sdsname_str, '.', 0)) == -1)
      strcpy(sds_name, sdsname_str);
    else   /* if specified */
    {
      len = (int)strlen(sdsname_str);
      strmid(sdsname_str, 0, p1, sds_name);
      p1++;
      if ((p2 = charpos(sdsname_str, '.', p1)) == -1)         /* 3D */
      {
        strmid(sdsname_str, p1, len-p1, nstr);
        *n = (int)atoi(nstr);
      }
      else                                                    /* 4D */
      {
        strmid(sdsname_str, p1, p2-p1, nstr);
        *n = (int)atoi(nstr);
        p2++;
        strmid(sdsname_str, p2, len-p2, nstr);
        *m = (int)atoi(nstr);
      }
    }
    if ((*n == 0) || (*m == 0))
    {
      fprintf(stderr, "Invalid layer number in SDS name: %s\n", sdsname_str);
      st = -1;
    }
    else  /* ZMS, 5/27/2019, because non -1 *n or *m is 1 based and needs to be converted to zero-based number */
    {
      if (*n != -1) --*n;
      if (*m != -1) --*m;
    }
  }
  return st;
}

int getlineL(FILE *fp, char *s)
{
  int c, i;

  c = fgetc(fp);
  for (i=0; i<MAX_LINE_LENGTH && c != EOF && c != '\n'; ++i)
  {
    s[i] = c;
    c = fgetc(fp);
  }
  s[i] = '\0';
  return i;
}

void split_string(char *tmp1_str, char **tmp_str, int *num)
{
  int p1, p2, len;

  p1 =  0;
  p2 = charpos(tmp1_str, ',', p1);
  while ((p2 != -1) && (*num < MAX_NUM_PARAM))
  {
    strmid(tmp1_str, p1, p2-p1, tmp_str[*num]);
    ++*num;
    p1 = p2 + 1;
    p2 = charpos(tmp1_str, ',', p1);
  }
  if (*num >= MAX_NUM_PARAM)
  {
    fprintf(stderr, "Too many parameters in option %s\n", tmp1_str);
    fprintf(stderr, "Considering only %d number of parameter values\n", MAX_NUM_PARAM);
  }
  else
  {
    len = (int)strlen(tmp1_str);
    if (p1 < len)
    {
      strmid(tmp1_str, p1, len-p1, tmp_str[*num]);
      ++*num;
    }
  }
}

/* MSZ copied stristr3 from 
 * https://stackoverflow.com/questions/27303062/strstr-function-like-that-ignores-upper-or-lower-case
 * but rename it to be strstr_case
 */
char* strstr_case(const char* haystack, const char* needle) {
  do {
    const char* h = haystack;
    const char* n = needle;
    while (tolower((unsigned char) *h) == tolower((unsigned char ) *n) && *n) {
      h++;
      n++;
    }
    if (*n == 0) {
      return (char *) haystack;
    }
  } while (*haystack++);
  return 0;
}

int strpos_wb(char *s1, char *s2, int p)
{
/* Maosheng Zhao developed on 4/12/2019 to make sure that the found string is word boundary */
  int fn, location;
  char *ptr = NULL;

  fn = 0 ;
  /* if s2 is found in s1 starting from the p location and also it is word boundary */
  if((ptr = strstr((char*)(s1 + p), s2)))
  {
    fn = 1;
    location = (int)(ptr - s1);

    /* the found string has word boundary in s1 */
    if(! ( (ptr - 1 >= s1 && isspace( *(ptr-1) ) ) && ( (ptr + strlen(s2) < &s1[strlen(s1)]) && isspace( *(ptr + strlen(s2)) ) ) ))
	fn = 0;
  }

  if (fn == 1) return location;
  else return -1; 
}
