/****************************************************************************
!C

!File: tileid.c

!Description:
  compute the tile id (horizontal vertical tile number)

!Input Parameters: (none)

!Input/Output Parameters: (none)

!Output Parameters: (none)

!Revision History:
  Original June 1999. QA Tool Version 1.2

! Revision History:
  Maosheng Zhao, SSAI, 4/3/2018 Version 1.3
  copy cproj.h, proj.h, isin.h and isinus.c from
  /LANDQA/f18005/f003/old_landqa1/qa1/QA_TOOLS/qa_tools_v2.0/croutines/lib
  to this src fold. also modfify the code by
  1, add #include "cproj.h"
  2, change
   int isinusint(double r, double center_long, double false_east, double false_north,
        int nzones, int nrows);
   int isinusfor(double lon, double lat, double *x, double *y);
      into 
   long isinusint(double r, double center_long, double false_east, double false_north,
        int nzones, int nrows);
   long isinusfor(double lon, double lat, double *x, double *y);

    to be consistent with the function declaration in proj.h

  3, the above free the tool from requiring the gctp version and just compile and link as
  gcc tileid.c isinus.c -o tileid -lm


!Team-unique Header:
  This software was developed by:
    Land Data Operational Product Evaluation (LDOPE) Team for the
    National Aeronautics and Space Administration, Goddard Space Flight
    Center

!References and Credits:
    Sadashiva Devadiga
    LDOPE                             Science Systems and Applications Inc.
    devadiga@ltpmail.gsfc.nasa.gov    NASA/GSFC Code 922
    phone: 301-614-5449               Greenbelt, MD 20771

!Design Notes: (none)

!END
*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "cproj.h"

#define HELP \
"NAME \n" \
"	tileid - compute the tile id (horizontal vertical tile number) \n" \
"\n" \
"SYNOPSIS \n" \
"	tileid [-help] \n" \
"	tileid <longitude> <latitude> \n" \
" \n" \
"       echo <longitude> <latitude> | tileid \n" \
" \n" \
"DESCRIPTION \n" \
"	This routine computes the horizontal and vertical tilenumber corresponding \n" \
" to the user specified geolocation in longitude and latitude \n" \
" \n" \
"OPTIONS \n" \
"	-help                       Display this help message \n" \
"	longitude                   longitude in degrees (-180 to +180) \n" \
"	latitude                    latitude in degrees (-90 to +90) \n" \
" \n" \
"Examples: \n" \
"	tileid -45.0 45.0 \n" \
" \n" \
"Last update: Version 3.6, 02/12/2020 \n" \
"Please report problems to Sadashiva Devadiga (sadashiva.devadiga-1@nasa.gov) \n"

#define USAGE \
"usage: tileid <longitude> <latitude> \n" \
" \n" \
"OPTIONS \n" \
"	-help                       Display this help message \n" \
"	longitude                   longitude in degrees (-180 to +180) \n" \
"	latitude                    latitude in degrees (-90 to +90) \n" \
" \n"

long isinusint(double r, double center_long, double false_east, double false_north,
        int nzones, int nrows);
long isinusfor(double lon, double lat, double *x, double *y);

int main(int argc, char *argv[])
{
  int st=0, hn, vn;
  double lon, lat, xproj, yproj;
   
  if ((argc == 2) && ((strcmp(argv[1], "-h")==0) || (strcmp(argv[1], "-help")==0)))
  {
    fprintf(stderr, "%s\n", HELP);
    exit(0);
  }
  if (argc != 3)
  {
    fprintf(stderr, "Incorrect of number of arguments input\n");
    fprintf(stderr, "%s\n", USAGE);
    exit(1);
  }
  
  lon = atof(argv[1]);
  lat = atof(argv[2]);
 
  if (fabs(lon) >= 180.0)
  {
    fprintf(stderr, "Invalid input longitude value: %f\n", lon);
    st = -1;
  }
      
  if (fabs(lat) >= 90.0)
  {
    fprintf(stderr, "Invalid input latitude value: %f\n", lat);
    st = -1;
  }
 
  if (st != -1)
  {
    isinusint(1.0, 0., 0., 0., 21600, 21600);
    isinusfor(lon, lat, &xproj, &yproj);
    xproj += 180.0;
    yproj = 90.0 - yproj;
    hn = (int)(xproj/10.0);
    vn = (int)(yproj/10.0);
    fprintf(stdout, "Tile ID (h, v): %d %d\n", hn, vn);
  }
  return 0;
}


